import CSSElement from './Abstract/CSSElement';
import Help from './Help';
import Validations from './Helpers/Validations';
import Conditions from './Helpers/Conditions';
import Replacements from './Helpers/Replacements';
import DynamicValue from './Helpers/DynamicValue';
/**
 * Field class.
 *
 * @since 1.0.0
 */
export default class Field extends CSSElement {
    fieldset;
    fields = [];
    type;
    name;
    label;
    prefix;
    suffix;
    placeholder;
    choices;
    conditions;
    validations;
    replacements;
    help;
    params;
    paramsCallback;
    valueDefault;
    valueCallback;
    value;
    dynamicValue;
    validated = false;
    inputClasses = [];
    /**
     * Initializing field.
     *
     * @param name  Name of the field.
     * @param field Field object
     *
     * @since 1.0.0
     */
    constructor(fieldset, field) {
        super();
        this.fieldset = fieldset;
        this.name = field.name;
        this.type = field.type;
        this.label = field.label;
        this.prefix = field.prefix;
        this.suffix = field.suffix;
        this.placeholder = field.placeholder;
        this.choices = field.choices;
        this.value = field.value;
        this.dynamicValue = new DynamicValue(this.fieldset.form, this.value, field.valueField !== undefined ? field.valueField : undefined, field.valueDefault !== undefined ? field.valueDefault : undefined, field.valueCallback !== undefined ? field.valueCallback : undefined);
        this.autoValue();
        this.validations = new Validations(this, field.validations !== undefined ? field.validations : []);
        this.replacements = new Replacements(this, field.replacements !== undefined ? field.replacements : []);
        this.conditions = new Conditions(this.fieldset.form, field.conditions !== undefined ? field.conditions : []);
        this.help = field.help === undefined ? undefined : new Help(field.help);
        field.classes?.forEach(className => this.addClass(className));
        field.fields?.forEach(subField => this.fields.push(new Field(this.fieldset, subField)));
        this.addClass('field-' + this.type);
    }
    /**
     * Sets autovalues for default or dynamic value if exists.
     *
     * @since 1.0.0
     */
    autoValue() {
        this.value = this.dynamicValue.get();
    }
    /**
     * Does field have choices.
     *
     * @return True if it has choices, false if not.
     *
     * @since 1.0.0
     */
    hasChoices() {
        return this.choices.length > 0;
    }
    /**
     * Validate the field.
     *
     * @return Array of errors, empty array on no errors.
     *
     * @since 1.0.0
     */
    validate() {
        if (!this.conditions.fullfilled())
            return []; // Do not validate if field is not shown.
        this.replacements.replace();
        this.validations.validate();
        if (this.validations.hasErrors()) {
            this.addClass('is-invalid');
            this.removeClass('is-valid');
        }
        else {
            this.addClass('is-valid');
            this.removeClass('is-invalid');
        }
        this.validated = true;
        return this.validations.getErrors();
    }
    /**
     * Has the field already ben validated
     *
     * @return True if field has been validated, false if not.
     *
     * @since 1.0.0
     */
    hasBeenValidated() {
        return this.validated;
    }
    /**
     * Add input class.
     *
     * @param string Name of css class to add.
     *
     * @since 1.0.0
     */
    addInputClass(name) {
        if (this.inputClasses.indexOf(name) !== -1) {
            return;
        }
        this.inputClasses.push(name);
    }
    /**
     * Remove input class.
     *
     * @param string Name of css class to remove.
     *
     * @since 1.0.0
     */
    removeInputClass(name) {
        this.inputClasses = this.inputClasses.filter(className => className !== name);
    }
    /**
     * Get input classes.
     *
     * @return String of CSS classes.
     *
     * @since 1.0.0
     */
    getInputClasses() {
        if (!this.hasBeenValidated()) {
            return this.inputClasses.join(' ');
        }
        if (!this.validations.hasErrors()) {
            this.addInputClass('is-valid');
            this.removeInputClass('is-invalid');
        }
        else {
            this.addInputClass('is-invalid');
            this.removeInputClass('is-valid');
        }
        return this.inputClasses.join(' ');
    }
}
